﻿Public Class SpaceTime
    Implements INotifyPropertyChanged
    Private Shared ReadOnly sizeUnits() As String = {"bytes", "K", "M", "G", "T"}
    Dim _space As Long
    Dim _time As TimeSpan

    Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged

    Public Property Space() As Long
        Set(ByVal value As Long)
            If _space <> value Then
                _space = value
                OnPropertyChanged(New PropertyChangedEventArgs("Space"))
                OnPropertyChanged(New PropertyChangedEventArgs("FormattedSpace"))
            End If
        End Set
        Get
            Return _space
        End Get
    End Property

    Public ReadOnly Property FormattedSpace() As String
        Get
            Return GetFormattedFileSize(Me.Space)
        End Get
    End Property

    Public Property Time() As TimeSpan
        Set(ByVal value As TimeSpan)
            If _time <> value Then
                _time = value
                OnPropertyChanged(New PropertyChangedEventArgs("Time"))
                OnPropertyChanged(New PropertyChangedEventArgs("FormattedTime"))
            End If
        End Set
        Get
            Return _time
        End Get
    End Property

    Public ReadOnly Property FormattedTime() As String
        Get
            Return GetFormattedDuration(Me.Time)
        End Get
    End Property

    Protected Overridable Sub OnPropertyChanged(ByVal args As PropertyChangedEventArgs)
        RaiseEvent PropertyChanged(Me, args)
    End Sub

    Public Shared Function GetFormattedFileSize(ByVal filesize As Long) As String
        Dim magnitude = 0
        Dim display As Double = 0
        Dim format = "G"

        If filesize > 0 Then
            Dim base2log = CInt(Math.Log(filesize) / Math.Log(2))
            magnitude = base2log \ 10
            display = filesize / Math.Pow(2, 10 * magnitude)
            Dim sigDigits = 1 + CInt(Math.Log10(display))
            format = "N" & Math.Max(0, 3 - sigDigits)
        End If
        Return display.ToString(format) & " "c & sizeUnits(magnitude)
    End Function

    Public Shared Function GetFormattedDuration(ByVal Duration As TimeSpan) As String
        If Duration.Days > 0 Then
            Dim hours = Duration.Hours + CInt((Duration.Minutes + Duration.Seconds \ 30) / 30)

            Return String.Format("{0} day{1} {2}hr{3}",
                                 Duration.Days,
                                 If(Duration.Days = 1, "", "s"),
                                 hours,
                                 If(hours = 1, "", "s"))

        ElseIf Duration.Hours > 0 Then
            Dim minutes = Duration.Minutes + Duration.Seconds \ 30

            Return String.Format("{0} hr{1} {2} min{3}",
                                 Duration.Hours,
                                 If(Duration.Hours = 1, "", "s"),
                                 minutes,
                                 If(minutes = 1, "", "s"))

        ElseIf Duration.Minutes > 0 Then
            Dim seconds = CInt(Math.Round(Duration.Seconds + Duration.Milliseconds / 1000.0))

            Return String.Format("{0} min{1} {2} sec{3}",
                                 Duration.Minutes,
                                 If(Duration.Minutes = 1, "", "s"),
                                 seconds,
                                 If(seconds = 1, "", "s"))

        ElseIf Duration.Seconds > 0 Then
            Dim seconds = Math.Round(Duration.Seconds + Duration.Milliseconds / 1000.0,
                                     If(Duration.Seconds < 10, 2, 1))

            Return String.Format("{0} second{1}", seconds, If(seconds = 1, "", "s"))
        End If

        Return String.Format("{0} msecs", Duration.Milliseconds)
    End Function
End Class
